﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectTourismCompany.Entities;
using ProjectTourismCompany.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Unity;

namespace ProjectTourismCompany.Repositories.Implementations;

internal class CompanyRepository : ICompanyRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<CompanyRepository> _logger;


    public CompanyRepository(IConnectionString connectionString, ILogger<CompanyRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }


    public void CreateCompany(Company company)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(company));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
                INSERT INTO Company (Name, CountryId)
                VALUES (@Name, @CountryId)";
            connection.Execute(queryInsert, company);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
        
    }

    public void DeleteCompany(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryDeleteSub = @"
            DELETE FROM CompanyTrip
            WHERE CompanyId = @id";
            connection.Execute(queryDeleteSub, new { id }, transaction);

            var queryDelete = @"
            DELETE FROM Company
            WHERE Id = @id";
            connection.Execute(queryDelete, new { id }, transaction);

            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }



    public IEnumerable<Company> ReadCompanies()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT co.*, coun.CountryName as CountryName FROM Company co LEFT JOIN Country coun on co.countryId = coun.Id";
            var peopleExpenses = connection.Query<Company>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(peopleExpenses));
            return peopleExpenses;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public Company ReadCompanyById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Company
                WHERE Id=@id";
            var expense = connection.QueryFirst<Company>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(expense));
            return expense;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public void UpdateCompany(Company company)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(company));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE Company
                SET
                Name=@Name,
                CountryId=@CountryId
                WHERE Id=@Id";
            connection.Execute(queryUpdate, company);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }
}
