﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using PdfSharp.Drawing;
using ProjectTourismCompany.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectTourismCompany.Repositories.Implementations;

internal class TripRepository : ITripRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<TripRepository> _logger;

    public TripRepository(IConnectionString connectionString, ILogger<TripRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }


    public void CreateTrip(Trip trip)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(trip));
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
                INSERT INTO Trip (Title, Price, HumanCapacity, StartDate, EndDate, StartCity, EndCity, TravelType)
                VALUES (@Title, @Price, @HumanCapacity, @StartDate, @EndDate, @StartCity, @EndCity, @TravelType);
                SELECT MAX(Id) FROM Trip";
            var TripId = connection.QueryFirst<int>(queryInsert, trip, transaction);
            
            
            var querySubInsert = @"
                INSERT INTO CompanyTrip (CompanyId, TripId, AdditionalPrice)
                VALUES (@CompanyId, @TripId, @AdditionalPrice)";
            foreach (var elem in trip.CompanyTrip)
            {
                connection.Execute(querySubInsert, new
                {
                    elem.Id,
                    elem.CompanyId,
                    TripId ,
                    elem.AdditionalPrice
                }, transaction);
            }
            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }

    }

    public Trip ReadTripById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Trip
                WHERE Id=@id";
            var expense = connection.QueryFirst<Trip>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}", JsonConvert.SerializeObject(expense));
            return expense;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public IEnumerable<Trip> ReadTrips(DateTime? dateFrom = null, DateTime? dateTo = null, int? clientId = null, int? companyId = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            var builder = new QueryBuilder();
            if (dateFrom.HasValue)
            {
                builder.AddCondition("t.StartDate >= @dateFrom");
            }
            if (dateTo.HasValue)
            {
                builder.AddCondition("t.EndDate <= @dateTo");
            }
            if (clientId.HasValue)
            {
                builder.AddCondition("t.ClientId = @clientId");
            }
            if (companyId.HasValue)
            {
                builder.AddCondition("ct.companyId = @companyId");
            }



            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @$"SELECT t.*, ct.tripId, ct.AdditionalPrice,comp.Name as CompanyName 
FROM Trip t 
INNER JOIN companyTrip ct ON ct.tripId = t.Id
Inner JOIN Company comp ON ct.companyId = comp.Id
{builder.Build()}";
            
            var tripDict = new Dictionary<int, List<CompanyTrip>>();

            
            var trips = connection.Query<Trip, CompanyTrip, Trip>(querySelect, (trip, companyTrip) =>
            {
                if (!tripDict.TryGetValue(trip.Id, out var ct))
                {
                    ct = [];
                    tripDict.Add(trip.Id, ct);
                }
                ct.Add(companyTrip);
                return trip;

            }, splitOn: "tripId", param: new { dateFrom , dateTo, clientId, companyId});
            _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(trips));
            return tripDict.Select(x =>
            {
                var tr = trips.First(y => y.Id == x.Key);
                tr.SetCompanyTrip(x.Value);
                return tr;
            }).ToArray();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }



}
